package sun.java2d.pipe;

import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImageOp;
import java.awt.image.ImageObserver;
import sun.awt.image.ToolkitImage;
import sun.java2d.loops.ScaledBlit;
import sun.java2d.loops.BlitBg;
import java.awt.image.WritableRaster;
import sun.awt.image.BytePackedRaster;
import java.awt.image.IndexColorModel;
import java.awt.image.ColorModel;
import java.awt.image.VolatileImage;
import sun.awt.image.SurfaceManager;
import sun.java2d.InvalidPipeException;
import sun.java2d.loops.Blit;
import sun.java2d.loops.CompositeType;
import sun.java2d.SurfaceData;
import sun.java2d.loops.MaskBlit;
import sun.java2d.loops.SurfaceType;
import sun.java2d.loops.TransformHelper;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.AlphaComposite;
import java.awt.image.BufferedImage;
import java.awt.geom.AffineTransform;
import java.awt.Color;
import java.awt.Image;
import sun.java2d.SunGraphics2D;

public class DrawImage implements DrawImagePipe
{
    private static final double MAX_TX_ERROR = 1.0E-4;
    
    public DrawImage() {
    }
    
    public boolean copyImage(SunGraphics2D sunGraphics2D, Image image, int n, int n2, Color color) {
        int width = image.getWidth(null);
        int height = image.getHeight(null);
        if (isSimpleTranslate(sunGraphics2D)) {
            return this.renderImageCopy(sunGraphics2D, image, color, n + sunGraphics2D.transX, n2 + sunGraphics2D.transY, 0, 0, width, height);
        }
        AffineTransform transform = sunGraphics2D.transform;
        if ((n | n2) != 0x0) {
            transform = new AffineTransform(transform);
            transform.translate((double)n, (double)n2);
        }
        this.transformImage(sunGraphics2D, image, transform, sunGraphics2D.interpolationType, 0, 0, width, height, color);
        return true;
    }
    
    public boolean copyImage(SunGraphics2D sunGraphics2D, Image image, int n, int n2, int n3, int n4, int n5, int n6, Color color) {
        if (isSimpleTranslate(sunGraphics2D)) {
            return this.renderImageCopy(sunGraphics2D, image, color, n + sunGraphics2D.transX, n2 + sunGraphics2D.transY, n3, n4, n5, n6);
        }
        this.scaleImage(sunGraphics2D, image, n, n2, n + n5, n2 + n6, n3, n4, n3 + n5, n4 + n6, color);
        return true;
    }
    
    public boolean scaleImage(SunGraphics2D sunGraphics2D, Image image, int n, int n2, int n3, int n4, Color color) {
        int width = image.getWidth(null);
        int height = image.getHeight(null);
        if (n3 > 0 && n4 > 0 && isSimpleTranslate(sunGraphics2D)) {
            double n5 = (double)(n + sunGraphics2D.transX);
            double n6 = (double)(n2 + sunGraphics2D.transY);
            if (this.renderImageScale(sunGraphics2D, image, color, sunGraphics2D.interpolationType, 0, 0, width, height, n5, n6, n5 + (double)n3, n6 + (double)n4)) {
                return true;
            }
        }
        AffineTransform transform = sunGraphics2D.transform;
        if ((n | n2) != 0x0 || n3 != width || n4 != height) {
            transform = new AffineTransform(transform);
            transform.translate((double)n, (double)n2);
            transform.scale((double)n3 / (double)width, (double)n4 / (double)height);
        }
        this.transformImage(sunGraphics2D, image, transform, sunGraphics2D.interpolationType, 0, 0, width, height, color);
        return true;
    }
    
    protected void transformImage(SunGraphics2D sunGraphics2D, Image image, int n, int n2, AffineTransform tx, int n3) {
        int type = tx.getType();
        int width = image.getWidth(null);
        int height = image.getHeight(null);
        int n4;
        if (sunGraphics2D.transformState <= 2 && (type == 0 || type == 1)) {
            double translateX = tx.getTranslateX();
            double translateY = tx.getTranslateY();
            double n9 = translateX + sunGraphics2D.transform.getTranslateX();
            double n10 = translateY + sunGraphics2D.transform.getTranslateY();
            int n11 = (int)Math.floor(n9 + 0.5);
            int n12 = (int)Math.floor(n10 + 0.5);
            if (n3 == 1 || (closeToInteger(n11, n9) && closeToInteger(n12, n10))) {
                this.renderImageCopy(sunGraphics2D, image, null, n + n11, n2 + n12, 0, 0, width, height);
                return;
            }
            n4 = false;
        }
        else if (sunGraphics2D.transformState <= 3 && (type & 0x78) == 0x0) {
            double[] array = { 0.0, 0.0, (double)width, (double)height };
            tx.transform(array, 0, array, 0, 2);
            int n5 = 0;
            array[n5] += (double)n;
            int n6 = 1;
            array[n6] += (double)n2;
            int n7 = 2;
            array[n7] += (double)n;
            int n8 = 3;
            array[n8] += (double)n2;
            sunGraphics2D.transform.transform(array, 0, array, 0, 2);
            if (this.tryCopyOrScale(sunGraphics2D, image, 0, 0, width, height, null, n3, array)) {
                return;
            }
            n4 = false;
        }
        else {
            n4 = true;
        }
        AffineTransform o = new AffineTransform(sunGraphics2D.transform);
        o.translate((double)n, (double)n2);
        o.concatenate(tx);
        if (!(n4 == 0)) {
            this.transformImage(sunGraphics2D, image, o, n3, 0, 0, width, height, null);
        }
        else {
            this.renderImageXform(sunGraphics2D, image, o, n3, 0, 0, width, height, null);
        }
    }
    
    protected void transformImage(SunGraphics2D sunGraphics2D, Image image, AffineTransform affineTransform, int n, int n2, int n3, int n4, int n5, Color color) {
        double[] array = { 0.0, 0.0, (double)(n4 - n2), 0.0, 0.0, 0.0 };
        array[3] = (array[5] = (double)(n5 - n3));
        affineTransform.transform(array, 0, array, 0, 3);
        if (Math.abs(array[0] - array[4]) < 1.0E-4 && Math.abs(array[3] - array[5]) < 1.0E-4 && this.tryCopyOrScale(sunGraphics2D, image, n2, n3, n4, n5, color, n, array)) {
            return;
        }
        this.renderImageXform(sunGraphics2D, image, affineTransform, n, n2, n3, n4, n5, color);
        return;
    }
    
    protected boolean tryCopyOrScale(SunGraphics2D sunGraphics2D, Image image, int n, int n2, int n3, int n4, Color color, int n5, double[] array) {
        double n6 = array[0];
        double n7 = array[1];
        double n8 = array[2];
        double n9 = array[3];
        double n10 = n8 - n6;
        double n11 = n9 - n7;
        if (n6 < -2.147483648E9 || n6 > 2.147483647E9 || n7 < -2.147483648E9 || n7 > 2.147483647E9 || n8 < -2.147483648E9 || n8 > 2.147483647E9 || n9 < -2.147483648E9 || n9 > 2.147483647E9) {
            return false;
        }
        if (closeToInteger(n3 - n, n10) && closeToInteger(n4 - n2, n11)) {
            int n12 = (int)Math.floor(n6 + 0.5);
            int n13 = (int)Math.floor(n7 + 0.5);
            if (n5 == 1 || (closeToInteger(n12, n6) && closeToInteger(n13, n7))) {
                this.renderImageCopy(sunGraphics2D, image, color, n12, n13, n, n2, n3 - n, n4 - n2);
                return true;
            }
        }
        return n10 > 0.0 && n11 > 0.0 && this.renderImageScale(sunGraphics2D, image, color, n5, n, n2, n3, n4, n6, n7, n8, n9);
    }
    
    BufferedImage makeBufferedImage(Image img, Color color, int imageType, int sx, int sy, int n, int n2) {
        int width = n - sx;
        int height = n2 - sy;
        BufferedImage bufferedImage = new BufferedImage(width, height, imageType);
        SunGraphics2D sunGraphics2D = (SunGraphics2D)bufferedImage.createGraphics();
        sunGraphics2D.setComposite(AlphaComposite.Src);
        bufferedImage.setAccelerationPriority(0.0f);
        if (color != null) {
            sunGraphics2D.setColor(color);
            sunGraphics2D.fillRect(0, 0, width, height);
            sunGraphics2D.setComposite(AlphaComposite.SrcOver);
        }
        sunGraphics2D.copyImage(img, 0, 0, sx, sy, width, height, null, null);
        sunGraphics2D.dispose();
        return bufferedImage;
    }
    
    protected void renderImageXform(SunGraphics2D sunGraphics2D, Image img, AffineTransform affineTransform, int n, int n2, int n3, int n4, int n5, Color bgColor) {
        AffineTransform inverse;
        try {
            inverse = affineTransform.createInverse();
        }
        catch (NoninvertibleTransformException ex) {
            return;
        }
        double[] array = new double[8];
        array[2] = (array[6] = (double)(n4 - n2));
        array[5] = (array[7] = (double)(n5 - n3));
        affineTransform.transform(array, 0, array, 0, 4);
        double a2;
        double a = a2 = array[0];
        double a4;
        double a3 = a4 = array[1];
        for (int i = 2; i < array.length; i += 2) {
            double n6 = array[i];
            if (a2 > n6) {
                a2 = n6;
            }
            else if (a < n6) {
                a = n6;
            }
            double n7 = array[i + 1];
            if (a4 > n7) {
                a4 = n7;
            }
            else if (a3 < n7) {
                a3 = n7;
            }
        }
        Region compClip = sunGraphics2D.getCompClip();
        int max = Math.max((int)Math.floor(a2), compClip.lox);
        int max2 = Math.max((int)Math.floor(a4), compClip.loy);
        int min = Math.min((int)Math.ceil(a), compClip.hix);
        int min2 = Math.min((int)Math.ceil(a3), compClip.hiy);
        if (min <= max || min2 <= max2) {
            return;
        }
        SurfaceData surfaceData = sunGraphics2D.surfaceData;
        SurfaceData surfaceData2 = surfaceData.getSourceSurfaceData(img, 4, sunGraphics2D.imageComp, bgColor);
        if (surfaceData2 == null) {
            img = this.getBufferedImage(img);
            surfaceData2 = surfaceData.getSourceSurfaceData(img, 4, sunGraphics2D.imageComp, bgColor);
            if (surfaceData2 == null) {
                return;
            }
        }
        if (isBgOperation(surfaceData2, bgColor)) {
            img = this.makeBufferedImage(img, bgColor, 1, n2, n3, n4, n5);
            n4 -= n2;
            n5 -= n3;
            n3 = (n2 = 0);
            surfaceData2 = surfaceData.getSourceSurfaceData(img, 4, sunGraphics2D.imageComp, bgColor);
        }
        TransformHelper transformHelper = TransformHelper.getFromCache(surfaceData2.getSurfaceType());
        if (transformHelper == null) {
            BufferedImage bufferedImage = this.makeBufferedImage(img, null, (surfaceData2.getTransparency() == 1) ? 1 : 2, n2, n3, n4, n5);
            n4 -= n2;
            n5 -= n3;
            n3 = (n2 = 0);
            surfaceData2 = surfaceData.getSourceSurfaceData(bufferedImage, 4, sunGraphics2D.imageComp, null);
            transformHelper = TransformHelper.getFromCache(surfaceData2.getSurfaceType());
        }
        SurfaceType surfaceType = surfaceData.getSurfaceType();
        if (sunGraphics2D.compositeState <= 1) {
            MaskBlit fromCache = MaskBlit.getFromCache(SurfaceType.IntArgbPre, sunGraphics2D.imageComp, surfaceType);
            if (fromCache.getNativePrim() != 0L) {
                transformHelper.Transform(fromCache, surfaceData2, surfaceData, sunGraphics2D.composite, compClip, inverse, n, n2, n3, n4, n5, max, max2, min, min2, null, 0, 0);
                return;
            }
        }
        int width = min - max;
        int height = min2 - max2;
        SurfaceData primarySurfaceData = SurfaceData.getPrimarySurfaceData(new BufferedImage(width, height, 3));
        SurfaceType surfaceType2 = primarySurfaceData.getSurfaceType();
        MaskBlit fromCache2 = MaskBlit.getFromCache(SurfaceType.IntArgbPre, CompositeType.SrcNoEa, surfaceType2);
        int[] array2 = new int[height * 2 + 2];
        transformHelper.Transform(fromCache2, surfaceData2, primarySurfaceData, AlphaComposite.Src, null, inverse, n, n2, n3, n4, n5, 0, 0, width, height, array2, max, max2);
        Blit.getFromCache(surfaceType2, sunGraphics2D.imageComp, surfaceType).Blit(primarySurfaceData, surfaceData, sunGraphics2D.composite, compClip.getIntersection(Region.getInstance(max, max2, min, min2, array2)), 0, 0, max, max2, width, height);
        return;
    }
    
    protected boolean renderImageCopy(SunGraphics2D sunGraphics2D, Image img, Color bgColor, int n, int n2, int n3, int n4, int n5, int n6) {
        Region region = sunGraphics2D.getCompClip();
        SurfaceData surfaceData = sunGraphics2D.surfaceData;
        int n7 = 0;
        while (true) {
            SurfaceData sourceSurfaceData = surfaceData.getSourceSurfaceData(img, 0, sunGraphics2D.imageComp, bgColor);
            if (sourceSurfaceData == null) {
                break;
            }
            try {
                this.blitSurfaceData(sunGraphics2D, region, sourceSurfaceData, surfaceData, sourceSurfaceData.getSurfaceType(), surfaceData.getSurfaceType(), n3, n4, n, n2, n5, n6, bgColor);
                return true;
            }
            catch (NullPointerException ex) {
                if (!SurfaceData.isNull(surfaceData) && !SurfaceData.isNull(sourceSurfaceData)) {
                    throw ex;
                }
                return false;
            }
            catch (InvalidPipeException ex2) {
                ++n7;
                region = sunGraphics2D.getCompClip();
                surfaceData = sunGraphics2D.surfaceData;
                if (SurfaceData.isNull(surfaceData) || SurfaceData.isNull(sourceSurfaceData) || n7 > 1) {
                    return false;
                }
                continue;
            }
        }
        return false;
    }
    
    protected boolean renderImageScale(SunGraphics2D sunGraphics2D, Image img, Color bgColor, int n, int n2, int n3, int n4, int n5, double n6, double n7, double n8, double n9) {
        if (n != 1) {
            return false;
        }
        Region region = sunGraphics2D.getCompClip();
        SurfaceData surfaceData = sunGraphics2D.surfaceData;
        int n10 = 0;
        while (true) {
            SurfaceData sourceSurfaceData = surfaceData.getSourceSurfaceData(img, 3, sunGraphics2D.imageComp, bgColor);
            if (sourceSurfaceData == null || isBgOperation(sourceSurfaceData, bgColor)) {
                break;
            }
            try {
                return this.scaleSurfaceData(sunGraphics2D, region, sourceSurfaceData, surfaceData, sourceSurfaceData.getSurfaceType(), surfaceData.getSurfaceType(), n2, n3, n4, n5, n6, n7, n8, n9);
            }
            catch (NullPointerException ex) {
                if (!SurfaceData.isNull(surfaceData)) {
                    throw ex;
                }
                return false;
            }
            catch (InvalidPipeException ex2) {
                ++n10;
                region = sunGraphics2D.getCompClip();
                surfaceData = sunGraphics2D.surfaceData;
                if (SurfaceData.isNull(surfaceData) || SurfaceData.isNull(sourceSurfaceData) || n10 > 1) {
                    return false;
                }
                continue;
            }
        }
        return false;
    }
    
    public boolean scaleImage(SunGraphics2D sunGraphics2D, Image image, int n, int n2, int n3, int n4, int n5, int n6, int n7, int n8, Color color) {
        boolean n9 = false;
        boolean n10 = false;
        boolean n11 = false;
        boolean n12 = false;
        int n13;
        int n14;
        if (n7 > n5) {
            n13 = n7 - n5;
            n14 = n5;
        }
        else {
            n9 = true;
            n13 = n5 - n7;
            n14 = n7;
        }
        int n15;
        int n16;
        if (n8 > n6) {
            n15 = n8 - n6;
            n16 = n6;
        }
        else {
            n10 = true;
            n15 = n6 - n8;
            n16 = n8;
        }
        int n17;
        if (n3 > n) {
            n17 = n3 - n;
        }
        else {
            n17 = n - n3;
            n11 = true;
            n = n3;
        }
        int n18;
        if (n4 > n2) {
            n18 = n4 - n2;
        }
        else {
            n18 = n2 - n4;
            n12 = true;
            n2 = n4;
        }
        if (n13 <= 0 || n15 <= 0) {
            return true;
        }
        if (n9 == n11 && n10 == n12 && isSimpleTranslate(sunGraphics2D)) {
            double n19 = (double)(n + sunGraphics2D.transX);
            double n20 = (double)(n2 + sunGraphics2D.transY);
            if (this.renderImageScale(sunGraphics2D, image, color, sunGraphics2D.interpolationType, n14, n16, n14 + n13, n16 + n15, n19, n20, n19 + (double)n17, n20 + (double)n18)) {
                return true;
            }
        }
        AffineTransform o = new AffineTransform(sunGraphics2D.transform);
        o.translate((double)n, (double)n2);
        o.scale((double)(n3 - n) / (double)(n7 - n5), (double)(n4 - n2) / (double)(n8 - n6));
        o.translate((double)(n14 - n5), (double)(n16 - n6));
        int imageScale = SurfaceManager.getImageScale(image);
        int n21 = image.getWidth(null) * imageScale;
        int n22 = image.getHeight(null) * imageScale;
        int n23 = n13 + n14;
        int n24 = n15 + n16;
        if (n23 > n21) {
            n23 = n21;
        }
        if (n24 > n22) {
            n24 = n22;
        }
        if (n14 < 0) {
            o.translate((double)(-n14), 0.0);
            n14 = 0;
        }
        if (n16 < 0) {
            o.translate(0.0, (double)(-n16));
            n16 = 0;
        }
        if (n14 >= n23 || n16 >= n24) {
            return true;
        }
        this.transformImage(sunGraphics2D, image, o, sunGraphics2D.interpolationType, n14, n16, n23, n24, color);
        return true;
    }
    
    public static boolean closeToInteger(int n, double n2) {
        return Math.abs(n2 - (double)n) < 1.0E-4;
    }
    
    public static boolean isSimpleTranslate(SunGraphics2D sunGraphics2D) {
        int transformState = sunGraphics2D.transformState;
        return transformState <= 1 || (transformState < 3 && sunGraphics2D.interpolationType == 1);
    }
    
    protected static boolean isBgOperation(SurfaceData surfaceData, Color color) {
        return surfaceData == null || (color != null && surfaceData.getTransparency() != 1);
    }
    
    protected BufferedImage getBufferedImage(Image image) {
        if ((image instanceof BufferedImage)) {
            return (BufferedImage)image;
        }
        return ((VolatileImage)image).getSnapshot();
    }
    
    private ColorModel getTransformColorModel(SunGraphics2D sunGraphics2D, BufferedImage bufferedImage, AffineTransform affineTransform) {
        ColorModel colorModel = bufferedImage.getColorModel();
        if (affineTransform.isIdentity()) {
            return colorModel;
        }
        int type = affineTransform.getType();
        int n = (type & 0x38) != 0x0;
        if (n == 0 && type != 1 && type != 0) {
            double[] flatmatrix = new double[4];
            affineTransform.getMatrix(flatmatrix);
            n = (flatmatrix[0] != (double)(int)flatmatrix[0] || flatmatrix[3] != (double)(int)flatmatrix[3]);
        }
        if (sunGraphics2D.renderHint != 2) {
            if ((colorModel instanceof IndexColorModel)) {
                WritableRaster raster = bufferedImage.getRaster();
                IndexColorModel indexColorModel = (IndexColorModel)colorModel;
                if (n && colorModel.getTransparency() == 1) {
                    if ((raster instanceof BytePackedRaster)) {
                        colorModel = ColorModel.getRGBdefault();
                    }
                    else {
                        double[] flatmatrix2 = new double[6];
                        affineTransform.getMatrix(flatmatrix2);
                        if (flatmatrix2[1] != 0.0 || flatmatrix2[2] != 0.0 || flatmatrix2[4] != 0.0 || flatmatrix2[5] != 0.0) {
                            int mapSize = indexColorModel.getMapSize();
                            if (mapSize < 256) {
                                int[] array = new int[mapSize + 1];
                                indexColorModel.getRGBs(array);
                                array[mapSize] = 0;
                                colorModel = new IndexColorModel(indexColorModel.getPixelSize(), mapSize + 1, array, 0, true, mapSize, 0);
                            }
                            else {
                                colorModel = ColorModel.getRGBdefault();
                            }
                        }
                    }
                }
            }
            else if (n && colorModel.getTransparency() == 1) {
                colorModel = ColorModel.getRGBdefault();
            }
        }
        else if (colorModel instanceof IndexColorModel || (n && colorModel.getTransparency() == 1)) {
            colorModel = ColorModel.getRGBdefault();
        }
        return colorModel;
    }
    
    protected void blitSurfaceData(SunGraphics2D sunGraphics2D, Region region, SurfaceData surfaceData, SurfaceData surfaceData2, SurfaceType surfaceType, SurfaceType surfaceType2, int n, int n2, int n3, int n4, int n5, int n6, Color color) {
        if (n5 <= 0 || n6 <= 0) {
            return;
        }
        CompositeType comp = sunGraphics2D.imageComp;
        if (CompositeType.SrcOverNoEa.equals(comp) && (surfaceData.getTransparency() == 1 || (color != null && color.getTransparency() == 1))) {
            comp = CompositeType.SrcNoEa;
        }
        if (!isBgOperation(surfaceData, color)) {
            Blit.getFromCache(surfaceType, comp, surfaceType2).Blit(surfaceData, surfaceData2, sunGraphics2D.composite, region, n, n2, n3, n4, n5, n6);
        }
        else {
            BlitBg.getFromCache(surfaceType, comp, surfaceType2).BlitBg(surfaceData, surfaceData2, sunGraphics2D.composite, region, color.getRGB(), n, n2, n3, n4, n5, n6);
        }
        return;
    }
    
    protected boolean scaleSurfaceData(SunGraphics2D sunGraphics2D, Region region, SurfaceData surfaceData, SurfaceData surfaceData2, SurfaceType src, SurfaceType dst, int n, int n2, int n3, int n4, double n5, double n6, double n7, double n8) {
        CompositeType compositeType = sunGraphics2D.imageComp;
        if (CompositeType.SrcOverNoEa.equals(compositeType) && surfaceData.getTransparency() == 1) {
            compositeType = CompositeType.SrcNoEa;
        }
        ScaledBlit fromCache = ScaledBlit.getFromCache(src, compositeType, dst);
        if (fromCache != null) {
            fromCache.Scale(surfaceData, surfaceData2, sunGraphics2D.composite, region, n, n2, n3, n4, n5, n6, n7, n8);
            return true;
        }
        return false;
    }
    
    protected static boolean imageReady(ToolkitImage toolkitImage, ImageObserver imageObserver) {
        if (toolkitImage.hasError()) {
            if (imageObserver != null) {
                imageObserver.imageUpdate(toolkitImage, 192, -1, -1, -1, -1);
            }
            return false;
        }
        else {
            return true;
        }
    }
    
    @Override
    public boolean copyImage(SunGraphics2D g, Image image, int x, int y, Color bg, ImageObserver iw) {
        if (!(image instanceof ToolkitImage)) {
            return this.copyImage(g, image, x, y, bg);
        }
        ToolkitImage img = (ToolkitImage)image;
        return imageReady(img, iw) && img.getImageRep().drawToBufImage(g, img, x, y, bg, iw);
    }
    
    @Override
    public boolean copyImage(SunGraphics2D g, Image image, int dx1, int dy1, int sx1, int sy1, int n, int n2, Color bg, ImageObserver iw) {
        if (!(image instanceof ToolkitImage)) {
            return this.copyImage(g, image, dx1, dy1, sx1, sy1, n, n2, bg);
        }
        ToolkitImage img = (ToolkitImage)image;
        return imageReady(img, iw) && img.getImageRep().drawToBufImage(g, img, dx1, dy1, dx1 + n, dy1 + n2, sx1, sy1, sx1 + n, sy1 + n2, bg, iw);
    }
    
    @Override
    public boolean scaleImage(SunGraphics2D g, Image image, int x, int y, int w, int h, Color bg, ImageObserver iw) {
        if (!(image instanceof ToolkitImage)) {
            return this.scaleImage(g, image, x, y, w, h, bg);
        }
        ToolkitImage img = (ToolkitImage)image;
        return imageReady(img, iw) && img.getImageRep().drawToBufImage(g, img, x, y, w, h, bg, iw);
    }
    
    @Override
    public boolean scaleImage(SunGraphics2D g, Image image, int dx1, int dy1, int dx2, int dy2, int sx1, int sy1, int sx2, int sy2, Color bg, ImageObserver iw) {
        if (!(image instanceof ToolkitImage)) {
            return this.scaleImage(g, image, dx1, dy1, dx2, dy2, sx1, sy1, sx2, sy2, bg);
        }
        ToolkitImage img = (ToolkitImage)image;
        return imageReady(img, iw) && img.getImageRep().drawToBufImage(g, img, dx1, dy1, dx2, dy2, sx1, sy1, sx2, sy2, bg, iw);
    }
    
    @Override
    public boolean transformImage(SunGraphics2D g, Image image, AffineTransform xform, ImageObserver iw) {
        if (!(image instanceof ToolkitImage)) {
            this.transformImage(g, image, 0, 0, xform, g.interpolationType);
            return true;
        }
        ToolkitImage img = (ToolkitImage)image;
        return imageReady(img, iw) && img.getImageRep().drawToBufImage(g, img, xform, iw);
    }
    
    @Override
    public void transformImage(SunGraphics2D sunGraphics2D, BufferedImage filter, BufferedImageOp bufferedImageOp, int n, int n2) {
        if (bufferedImageOp != null) {
            if ((bufferedImageOp instanceof AffineTransformOp)) {
                AffineTransformOp affineTransformOp = (AffineTransformOp)bufferedImageOp;
                this.transformImage(sunGraphics2D, filter, n, n2, affineTransformOp.getTransform(), affineTransformOp.getInterpolationType());
                return;
            }
            filter = bufferedImageOp.filter(filter, null);
        }
        this.copyImage(sunGraphics2D, filter, n, n2, null);
    }
}
